<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 12/03/2004 *
 * ===================                          Last change: 12/13/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : theme manager.php                                *
 * -------------------------------------------------------------------- *
 * Short description : Theme manager                                    *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Themen-Manager                                   *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009, 2010 by Mailer Developer Team                    *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Create a selection box with installed and activated themes or all if admin
function generateThemeSelectionBox () {
	// Init variables and fill them if set
	$what = getWhat();
	$mod = getModule();

	// Construction URL
	$formAction = '{%url=modules.php?module='. $mod;
	if (!empty($what)) $formAction .= '&amp;what=' . $what;
	$formAction .= '%}';

	// Initialize array
	$themesArray = array(
		'theme_unix'   => array(), // Unix name from filesystem
		'theme_name'   => array()  // Title
	);

	// Only activated themes for the user
	$add = " WHERE `theme_active`='Y'";

	// Do we have admin?
	if (isAdmin()) $add = '';

	// Select all themes we want
	$result = SQL_QUERY("SELECT
	`theme_path`, `theme_name`
FROM
	`{?_MYSQL_PREFIX?}_themes`
".$add."
ORDER BY
	`theme_name` ASC", __FUNCTION__, __LINE__);

	// Load all themes
	while ($content = SQL_FETCHARRAY($result)) {
		// Construct relative include file name
		$inc = sprintf("theme/%s/theme.php", secureString($content['theme_path']));

		// Load it's theme.php file if found
		if (isIncludeReadable($inc)) {
			// And save all data in array
			loadInclude($inc);
			$themesArray['theme_unix'][] = $content['theme_path'];
			$themesArray['theme_name'][] = $GLOBALS['theme_data']['name'];
		} // END - if
	} // END - while

	// Free the result
	SQL_FREERESULT($result);

	// Construct selection form for the box template
	// @TODO Can't this be rewritten to an API function?
	$OUT = '';
	foreach ($themesArray['theme_unix'] as $key => $theme) {
		$OUT .= '  <option value="' . $theme . '"';
		if ($theme == getCurrentTheme()) $OUT .= ' selected="selected"';
		$OUT .= '>' . $themesArray['theme_name'][$key] . '</option>';
	} // END - foreach

	// Remember content
	$content = array(
		'form_action' => $formAction,
		'selection'   => $OUT
	);

	// Return generated selection
	return loadTemplate('theme_select_form', true, $content);
}

// Get version from name
function getThemeVersion ($name) {
	// Is the extension 'theme' installed?
	if (!isExtensionActive('theme')) {
		// Then abort here
		return '!.!';
	} // END - if

	// Default version 'number'
	$cver = '?.?';

	// Is the cache entry there?
	if (isset($GLOBALS['cache_array']['themes']['theme_version'][$name])) {
		// Get the version from cache
		$cver = $GLOBALS['cache_array']['themes']['theme_version'][$name];

		// Count up
		incrementStatsEntry('cache_hits');
	} elseif (isExtensionInstalledAndNewer('cache', '0.1.8')) {
		// Load version from database
		$result = SQL_QUERY_ESC("SELECT `theme_ver` FROM `{?_MYSQL_PREFIX?}_themes` WHERE `theme_path`='%s' LIMIT 1",
			array($name), __FUNCTION__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($cver) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	}

	// Return version
	return $cver;
}

// Checks wether a theme is found in db
function ifThemeExists ($name) {
	// Get theme and is it not nul?
	return ((isExtensionActive('theme')) && (getThemeId($name) > 0));
}

// Checks if a theme is active
function isThemeActive ($name) {
	// Is the extension 'theme' installed?
	if (!isExtensionActive('theme')) {
		// Then abort here
		return false;
	} // END - if

	// Default is nothing active
	$active = false;

	// Is the cache entry there?
	if (isset($GLOBALS['cache_array']['themes']['theme_active'][$name])) {
		// Get the version from cache
		$active = ($GLOBALS['cache_array']['themes']['theme_active'][$name] == 'Y');

		// Count up
		incrementStatsEntry('cache_hits');
	} elseif (isExtensionInstalledAndNewer('cache', '0.1.8')) {
		// Check if current theme is already imported or not
		$result = SQL_QUERY_ESC("SELECT `theme_active` FROM `{?_MYSQL_PREFIX?}_themes` WHERE `theme_path`='%s' AND `theme_active`='Y' LIMIT 1",
			array($name), __FUNCTION__, __LINE__);

		// Is the theme active and installed?
		$active = (SQL_NUMROWS($result) == 1);

		// Free result
		SQL_FREERESULT($result);
	}

	// Return result
	return $active;
}

// Gets current human-readable theme name
function getCurrentThemeName () {
	// Is the extension 'theme' installed?
	if (!isExtensionActive('theme')) {
		// Then abort here
		return 'default';
	} // END - if

	// Get the Uni* name
	$name = getCurrentTheme();

	// Is the cache entry there?
	if (isset($GLOBALS['cache_array']['themes']['theme_name'][$name])) {
		// Get the version from cache
		$name = $GLOBALS['cache_array']['themes']['theme_name'][$name];

		// Count up
		incrementStatsEntry('cache_hits');
	} elseif (isExtensionInstalledAndNewer('cache', '0.1.8')) {
		// Check if current theme is already imported or not
		$result = SQL_QUERY_ESC("SELECT `theme_name` FROM `{?_MYSQL_PREFIX?}_themes` WHERE `theme_path`='%s' AND `theme_active`='Y' LIMIT 1",
			array($name), __FUNCTION__, __LINE__);

		// Load theme name
		list($name) = SQL_FETCHROW($result);

		// Free result
		SQL_FREERESULT($result);
	}

	// Return name
	return $name;
}

// Get current theme name
function getActualTheme () {
	// The default theme is 'default'... ;-)
	$ret = 'default';

	// Load default theme if not empty from configuration
	if ((isConfigEntrySet('default_theme')) && (getConfig('default_theme') != '')) $ret = getConfig('default_theme');

	if (!isSessionVariableSet('mxchange_theme')) {
		// Set default theme
		setTheme($ret);
	} elseif ((isSessionVariableSet('mxchange_theme')) && (isExtensionInstalledAndNewer('sql_patches', '0.1.4'))) {
		//die("<pre>".print_r($GLOBALS['cache_array']['themes'], true)."</pre>");
		// Get theme from cookie
		$ret = getSession('mxchange_theme');

		// Is it valid?
		if ((!isExtensionActive('theme')) || (getThemeId($ret) == '0')) {
			// Fix it to default
			$ret = 'default';
		} // END - if
	} elseif ((!isInstalled()) && ((isInstalling()) || (getOutputMode() == true)) && ((isGetRequestParameterSet('theme')) || (isPostRequestParameterSet('theme')))) {
		// Prepare filename for checking
		$themeFile = sprintf("theme/%s/theme.php", getRequestParameter('theme'));

		// Installation mode active
		if ((isGetRequestParameterSet('theme')) && (isIncludeReadable($theme))) {
			// Set cookie from URL data
			setTheme(getRequestParameter('theme'));
		} elseif (isIncludeReadable(sprintf("theme/%s/theme.php", secureString(postRequestParameter('theme'))))) {
			// Set cookie from posted data
			setTheme(secureString(postRequestParameter('theme')));
		}

		// Set return value
		$ret = getSession('mxchange_theme');
	} else {
		// Invalid design, reset cookie
		setTheme($ret);
	}

	// Return theme value
	return $ret;
}

// Setter for theme in session
function setTheme ($newTheme) {
	setSession('mxchange_theme', $newTheme);
}

// Get id from theme
function getThemeId ($name) {
	// Default id
	$id = '0';

	// Is the cache entry there?
	if (isset($GLOBALS['cache_array']['themes']['id'][$name])) {
		// Get the version from cache
		$id = $GLOBALS['cache_array']['themes']['id'][$name];

		// Count up
		incrementStatsEntry('cache_hits');
	} elseif (isExtensionInstalledAndNewer('cache', '0.1.8')) {
		// Check if current theme is already imported or not
		$result = SQL_QUERY_ESC("SELECT `id` FROM `{?_MYSQL_PREFIX?}_themes` WHERE `theme_path`='%s' LIMIT 1",
			array($name), __FUNCTION__, __LINE__);

		// Entry found?
		if (SQL_NUMROWS($result) == 1) {
			// Fetch data
			list($id) = SQL_FETCHROW($result);
		} // END - if

		// Free result
		SQL_FREERESULT($result);
	}

	// Return id
	return $id;
}

///////////////////////////////////////////////////////////////////////////////
//                              Only filter functions
///////////////////////////////////////////////////////////////////////////////

// Filter for generic handling of theme change
function FILTER_HANDLE_THEME_CHANGE () {
	// Check if new theme is selcted
	if ((isPostRequestParameterSet('new_theme')) && (postRequestParameter('new_theme') != getCurrentTheme())) {
		// Set new theme for guests
		$newTheme = postRequestParameter('new_theme');

		// Change to new theme
		setTheme($newTheme);

		// Remove current from array
		removeIncludeFromPool('theme', sprintf("theme/%s/theme.php", getCurrentTheme()));

		// Add new theme
		addIncludeToPool('theme', sprintf("theme/%s/theme.php", $newTheme));

		// Redirect to same URL
		redirectToRequestUri();
	} // END - if
}

// Filter for settings theme from user profile, must be executed only if FILTER_FETCH_USER_DATA() ran before
function FILTER_SET_USERS_THEME () {
	// Is the user data valid?
	if (!isMember()) {
		// Do only run for logged in members
		debug_report_bug(__FUNCTION__, __LINE__, 'Please only run this filter for logged in users.');
	} // END - if

	// Change to new theme
	setTheme(getUserData('curr_theme'));
}

// [EOF]
?>
